# Moderation on Kubernetes Communications Channels

This page describes the rules and best practices for people chosen to moderate Kubernetes communications channels. 
This includes, Slack and the mailing lists and _any communication tool_ used in an official manner by the project. 

- Check the [centralized list of administrators](./moderators.md) for contact information.

## Roles and Responsibilities

As part of volunteering to become a moderator you are now representative of the Kubernetes community and it is your responsibility to remain aware of your contributions in this space. 
These responsibilities apply to all Kubernetes official channels. 

Moderators _MUST_:   

- Take action as specified by these Kubernetes Moderator Guidelines.
  - You are empowered to take _immediate action_ when there is a violation. You do not need to wait for review or approval if an egregious violation has occurred. Make a judgement call based on our Code of Conduct and Values (see below). 
  - Removing a bad actor or content from the medium is preferable to letting it sit there. 
- Abide by the documented tasks and actions required of moderators.
- Ensure that the [CNCF Code of Conduct](https://github.com/cncf/foundation/blob/master/code-of-conduct.md) is in effect on all official Kubernetes communication channels.
- Become familiar with the [Kubernetes Community Values](https://github.com/kubernetes/steering/blob/master/values.md).
- Take care of spam as soon as possible, which may mean taking action by removing a member from that resource.
- Foster a safe and productive environment by being aware of potential multiple cultural differences between Kubernetes community members.
- Understand that you might be contacted by moderators, community managers, and other users via private email or a direct message. 
- Report violations of the Code of Conduct to <conduct@kubernetes.io>.

Moderators _SHOULD_: 

- Exercise compassion and empathy when communicating and collaborating with other community members.
- Understand the difference between a user abusing the resource or just having difficulty expressing comments and questions in English.
- Be an example and role model to others in the community.
- Remember to check and recognize if you need take a break when you become frustrated or find yourself in a heated debate.
- Help your colleagues if you recognize them in one of the [stages of burnout](https://opensource.com/business/15/12/avoid-burnout-live-happy).
- Be helpful and have fun! 

## Violations

The Kubernetes [Code of Conduct Committee](https://git.k8s.io/community/committee-code-of-conduct) will have the final authority regarding escalated moderation matters.  Violations of the Code of Conduct will be handled on a case by case basis. Depending on severity this can range up to and including removal of the person from the community, though this is extremely rare.

## Specific Guidelines

These guidelines are for tool-specific policies that don't fit under a general umbrella. 

### Mailing Lists

### Moderating a SIG/WG list

- SIG and Working Group mailing list should have parispittman@google.com and jorgec@vmware.com as a coowner so that administrative functions can be managed centrally across the project.
  - Moderation of the SIG/WG lists is up to that individual SIG/WG, these admins are there to help facilitate leadership changes, reset lost passwords, etc. 

- Users who are violating the Code of Conduct or other negative activities (like spamming) should be moderated.
  - [Lock the thread immediately](https://support.google.com/groups/answer/2466386?hl=en#) so that people cannot reply to the thread.
  - [Delete the post](https://support.google.com/groups/answer/1046523?hl=en) - 
  - In some cases you might need to ban a user from the group, follow [these instructions](https://support.google.com/groups/answer/2646833?hl=en&ref_topic=2458761#) on how stop a member from being able to post to the group. 

For more technical help on how to use Google Groups, check the [Groups Help](https://support.google.com/groups/answer/2466386?hl=en&ref_topic=2458761) page.

### New users posting to a SIG/WG list
New members who post to a group will automatically have their messages put in a queue and be sent the following message automatically: "Since you're a new subscriber you're in a moderation queue, sorry for the inconvenience, a moderator will check your message shortly."

Moderators will receive emails when messages are in this queue and will process them accordingly.

### Discuss

- [Discuss Guidelines](./discuss-guidelines.md)

### Slack

- [Slack Guidelines](./slack-guidelines.md)

### Zoom 

- [Zoom Guidelines](./zoom-guidelines.md)


### References and Resources

Thanks to the following projects for making their moderation guidelines public, allowing us to build on the shoulders of giants.
Moderators are encouraged to learn how other projects moderate and learn from them in order to improve our guidelines:

- Mozilla's [Forum Moderation Guidelines](https://support.mozilla.org/en-US/kb/moderation-guidelines)
- OASIS [How to Moderate a Mailing List](https://www.oasis-open.org/khelp/kmlm/user_help/html/mailing_list_moderation.html)
- Community Spark's [How to effectively moderate forums](http://www.communityspark.com/how-to-effectively-moderate-forums/)
- [5 tips for more effective community moderation](https://www.socialmediatoday.com/social-business/5-tips-more-effective-community-moderation)
- [8 Helpful Moderation Tips for Community Managers](https://sproutsocial.com/insights/tips-community-managers/)
- [Setting Up Community Guidelines for Moderation](https://www.getopensocial.com/blog/community-management/setting-community-guidelines-moderation)
